;;; @file basic.s
;;; Interrupt-driven file server for 8-bit Commodores, with BASIC loader
;;; @author Marko Mkel (msmakela@nic.funet.fi)

#include "common.s"

	.text
	.(
	.word basicstart
	*=basicstart
	.word nextln
	.word host
#if basicstart = 3		; PET II series
codediff= serverstart - codestart
codeend = codestart + reloclength - 1
	.byte $81, $49, $b2	; FOR I=
	.byte $30 + (codestart / 10)
	.byte $30 + (codestart - (codestart / 10 * 10))
	.byte $a4		; TO
	.byte $30 + (codeend / 100)
	.byte $30 + ((codeend - (codeend / 100 * 100)) / 10)
	.byte $30 + (codeend - (codeend / 10 * 10))
	.byte ":"
	.byte $dc, $30 + bbank	; BANK
	.byte ":J"
	.byte $b2, $c2, "(I):"	; J=PEEK(I):
	.byte $dc, "15:"	; BANK 15:
	.byte $97, "I", $aa	; POKE I+
	.byte $30 + (codediff / 100)
	.byte $30 + ((codediff - (codediff / 100 * 100)) / 10)
	.byte $30 + (codediff - (codediff / 10 * 10))
	.byte ",J:"		; ,J:
	.byte $82, ":"		; NEXT:
	.byte $9e		; SYS
	.byte $30 + (serverstart / 1000)
	.byte $30 + ((serverstart - (serverstart / 1000 * 1000)) / 100)
	.byte $30 + ((serverstart - (serverstart / 100 * 100)) / 10)
	.byte $30 + (serverstart - (serverstart / 10 * 10))
	.byte ":"
	.byte $a2		; NEW
	.byte 0
nextln	.word 0
codestart
	*=serverstart
#else				; PET II series
loffset	= <lstart
	.byte $9e		; SYS
	.byte $30 + (loffset / 10)
	.byte $30 + (loffset - (loffset / 10 * 10))
	.byte $aa, "256", $ac	; + 256 *
	.byte $c2, "("		; PEEK
	.byte $30 + ((bstart + 1) / 10)
	.byte $30 + ((bstart + 1) - ((bstart + 1) / 10 * 10))
	.byte "):"
	.byte $a2		; NEW
	.byte 0
#if serverstart - (serverstart / 256 * 256)
#error				; the server must start at $xx00
#endif
#if relocstart - (relocstart / 256 * 256)
#error				; the server must start at $xx00
#endif
nextln	.word lastln
targetpage = *+1
	.word serverstart
	.byte $9e		 ; SYS
	.byte 0
lastln	.word 0
lstart	ldy #0
	lda bstart+1
	sty end
	sta end+1
	ldx #<codestart
	stx start
	sta start+1
	ldy #<copy+2		; patch the calls to cp
	sta (end),y
	ldy #<copy2+2
	sta (end),y
	ldy #<targetpage
	lda (end),y		; get the relocation address
	ldy #<targetpage2
	sta (end),y		; store the relocation address
	ldy #<launch+2
	sta (end),y		; and the jump address
	sta end+1		; store the destination address for copying
	;; see if the BASIC area needs to be truncated
	cmp memsiz+1		; destination below end of BASIC?
	bcs docopy
	cmp bstart+1		; destination above start of BASIC?
	bcc docopy
	sta memsiz+1		; limit the end of BASIC
	;; copy and relocate the server
docopy	ldy #0
	ldx #>reloclength	; copy the first n*256 bytes
copy	jsr cp
	bne copy
	inc start+1
	inc end+1
	dex
	bne copy
copy2	jsr cp			; copy the last 0..255 bytes
	cpy #<reloclength
	bcc copy2
launch	jmp serverstart

	;; *end++ = relocate (*start++)
cp	lda (start),y
	cmp #>relocated
	bcc write
	cmp #>relocated+reloclength+256
	bcs write
	sbc #>relocated-256	; C=0 after cmp
	clc
targetpage2 = *+1
	adc #>serverstart
write	sta (end),y
	iny
	rts

codestart
#if codestart - basicstart > 254
#error				; relocator code too long
#endif
	;; All servers are less than one kilobyte long, and they
	;; start at a 256-byte boundary.  Thus, there are up to
	;; four consecutive address bytes that need patching.
	;; These bytes should not occur anywhere else in the code.
	;; This is why the servers are assembled at a different
	;; address.  Suitable addresses can be sought with the
	;; following command:
	;; find ../cbmprg -type f -print | xargs perl reladdr
	;; On the Commodore 64, the address is different for each
	;; type of memory expansion.
	*=relocstart
#ifdef reu_aux
#ifdef pia_init
#ifdef relocstart_piareu
	*=relocstart_piareu
#endif
#endif
#ifndef pia_init
#ifdef relocstart_reu
	*=relocstart_reu
#endif
#endif
#endif
#ifndef reu_aux
#ifdef pia_init
#ifdef relocstart_pia
	*=relocstart_pia
#endif
#endif
#endif
#endif				 ; PET II series
	.)

relocated
#include "main.s"
reloclength = *-relocated
